@@ -21,7 +21,7 @@
 
 networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
                   edge.spells=NULL,vertex.spells=NULL,edge.changes=NULL,vertex.changes=NULL,
-                  network.list=NULL,onsets=NULL,termini=NULL,vertex.pid=NULL,start=NULL,end=NULL,net.obs.period=NULL,verbose=TRUE,...) {
+                  network.list=NULL,onsets=NULL,termini=NULL,vertex.pid=NULL,start=NULL,end=NULL,net.obs.period=NULL,verbose=TRUE,create.TEAs=FALSE,...) {
   
   
   if (!is.null(start) && !is.null(end)) {
@@ -115,7 +115,30 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
       }
         
     } 
-       
+    copyTEAs<-create.TEAs #flag to control if we process TEAs, which is slow
+    if (copyTEAs){
+      # construct the list of vertex attributes that may become TEAs
+      TEAvertAttrs<-unique(unlist(lapply(network.list,list.vertex.attributes)))
+      TEAvertAttrs<-TEAvertAttrs[!TEAvertAttrs%in%c('na','vertex.names',vertex.pid)]
+      # also for edges
+      TEAedgeAttrs<-unique(unlist(lapply(network.list,list.edge.attributes)))
+      TEAedgeAttrs<-TEAedgeAttrs[!TEAedgeAttrs%in%c('na')]
+      # and for networks
+      TEAnetAttrs<-unique(unlist(lapply(network.list,list.network.attributes)))
+      TEAnetAttrs<-TEAnetAttrs[!TEAnetAttrs%in%c('directed', 'hyper', 'loops', 'multiple', 'bipartite','mnext','net.obs.period','vertex.pid','edge.pid','n')]
+      if (verbose){
+        cat("Dynamic attributes (TEAs) will be created for the following attributes of the networks on network.list:\n")
+        if (length(TEAnetAttrs)){
+          cat("\tNetwork: ",TEAnetAttrs,"\n")
+        }
+        if (length(TEAvertAttrs)){
+          cat("\tVertices: ",TEAvertAttrs,"\n")
+        }
+        if (length(TEAedgeAttrs)){
+          cat("\tEdges: ",TEAedgeAttrs,"\n")
+        }
+      }
+    }
     
     # ---- vertex.pid present ----
     if (!is.null(vertex.pid)) {
@@ -134,11 +157,24 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
       base.net.pids <- sort(unique(base.net.pids))
       
       # initialize, copy network attributes
+      vattrs=character(0)
+      nattrs=character(0)
       if (is.null(base.net)){ 
         base.net <- network.list[[1]]
         if (verbose){
           cat("Argument base.net not specified, using first element of network.list instead\n")
         }
+        # attributes will not be copied from base net
+      } else {
+        # get the list of attributes to copy from 
+        vattrs<-list.vertex.attributes(base.net)
+        if (copyTEAs){
+          # don't create a TEA if attribute of the same name exists only in base net
+          TEAvertAttrs<-setdiff(TEAvertAttrs,vattrs)
+        }
+        
+        nattrs<-list.network.attributes(base.net)
+        nattrs<-setdiff(nattrs,c("bipartite","directed","hyper","loops","mnext","multiple","n"))
       }
       # todo: should we warn here that used first network as base net?
       # I guess the reason to init instead of copy is in case the pids imply a different network size?
@@ -148,14 +184,12 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
       if (verbose){
         cat(paste("Initialized network of size",network.size(out.net),"inferred from number of unique vertex.pids\n"))
       }
-      # copy vertex attributes
-      vattrs<-list.vertex.attributes(base.net)
+      # copy vertex attributes (only occurs if real base net specified)
+      
       for(attr in vattrs){
         set.vertex.attribute(out.net,attr,get.vertex.attribute(base.net,attr))
       }
-      # copy network attributes EXCEPT network properties
-      nattrs<-list.network.attributes(base.net)
-      nattrs<-setdiff(nattrs,c("bipartite","directed","hyper","loops","mnext","multiple","n"))
+      # copy network attributes EXCEPT network properties (if real base.net specified)
       for (attr in nattrs){
         set.network.attribute(out.net,attr,get.network.attribute(base.net,attr))
       }
@@ -166,7 +200,7 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
       set.network.attribute(out.net,'vertex.pid',vertex.pid)
       
       base.net <- out.net
-      
+        
       # get combined edge list, indexed by the vertices in base.net
       for (i in seq_along(network.list)) {
         edgelist <- as.edgelist(network.list[[i]])
@@ -182,6 +216,13 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
         ts<-rep(termini[i], length(vertices))
         activate.vertices(base.net,onset=os,terminus=ts,v=vertices)
         
+        if (copyTEAs){
+          # copy any non-standard, vertex, attributes into TEAs
+          for(attr in TEAvertAttrs){
+            activate.vertex.attribute(base.net,attr,get.vertex.attribute(network.list[[i]],attr,unlist=FALSE),onset=onsets[i],terminus=termini[i],v=get.vertex.id(base.net,net.pids))
+          }
+        }
+        
         # activate the edges
         for (e in seq_len(nrow(edges))) {
           t <- edges[e,1]
@@ -190,13 +231,29 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
           if (length(get.edgeIDs(base.net, t, h)) == 0) {
             add.edge(base.net, tail=t, head=h)
           }
-          activate.edges(base.net, e = get.edgeIDs(base.net,t,h)[1], 
-                         onset=onsets[i], terminus=termini[i]) 
+          eid<-get.edgeIDs(base.net,t,h)[1]
+          activate.edges(base.net, e = eid,  onset=onsets[i], terminus=termini[i])
+          if (copyTEAs){
+            #  copy any non-standard edge attributes into TEAs
+            for(attr in TEAedgeAttrs){
+              activate.edge.attribute(base.net,attr,get.edge.attribute(network.list[[i]],attrname=attr,unlist=FALSE)[get.edgeIDs(network.list[[i]],v=edgelist[e,1],alter=edgelist[e,2])],e=eid,onset=onsets[i],terminus=termini[i])
+            }
+          }
           
+        } # end edge copy loop
+        
+        if (copyTEAs){
+          # copy any non-standard network attributes into TEAs
+          for(attr in TEAnetAttrs){
+            activate.network.attribute(base.net,attr,get.network.attribute(network.list[[i]],attr,unlist=FALSE),onset=onsets[i],terminus=termini[i])
+          }
         }
         
-        # TODO: copy any non-standard, vertex, edge, or network attributs into TEAs
-      }
+      } # end network loop
+      
+      
+      
+      
       
     } else {
       # ---- no vertex.pid, all networks same size ----
@@ -209,7 +266,7 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
       out.net <- network.initialize(network.size(base.net), directed = base.net%n%"directed", 
                                    hyper = base.net%n%"hyper", loops = base.net%n%"loops", 
                                    multiple = base.net%n%"multiple", bipartite = base.net%n%"bipartite")
-      # copy vertex attributes
+      # copy vertex attributes  These may be overwritten by TEAs if they appear on later networks
       vattrs<-list.vertex.attributes(base.net)
       for(attr in vattrs){
         set.vertex.attribute(out.net,attr,get.vertex.attribute(base.net,attr))
@@ -226,11 +283,15 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
       for (i in seq_along(network.list)) {
         edges <- as.edgelist(network.list[[i]])
         
-        
         # all vertices are assumed to be active
         activate.vertices(base.net, onset=onsets[i], terminus=termini[i])
         
-        # TODO: copy any vertex attributes into a vertex TEA
+        if(copyTEAs){
+          # copy any vertex attributes into a vertex TEA
+          for(attr in TEAvertAttrs){
+            activate.vertex.attribute(base.net,attr,get.vertex.attribute(network.list[[i]],attr,unlist=FALSE),onset=onsets[i],terminus=termini[i])
+          }
+        }
         
         # activate the edges
         for (e in seq_len(nrow(edges))) {
@@ -240,15 +301,25 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
           if (length(get.edgeIDs(base.net, t, h)) == 0) {
             add.edge(base.net, tail=t, head=h)
           }
-          activate.edges(base.net, e = get.edgeIDs(base.net,t,h)[1], 
-                         onset=onsets[i], terminus=termini[i]) 
-          
+          eid <- get.edgeIDs(base.net,t,h)[1]
+          activate.edges(base.net, e = eid, onset=onsets[i], terminus=termini[i]) 
+          if (copyTEAs){
+            # copy any edge attributes into an edge TEA
+            for(attr in TEAedgeAttrs){
+              activate.edge.attribute(base.net,attr,get.edge.attribute(network.list[[i]],attrname=attr,unlist=FALSE)[get.edgeIDs(network.list[[i]],v=t,alter=h)],e=eid,onset=onsets[i],terminus=termini[i])
+            }
+          }
+        }
+        if (copyTEAs){
+          # copy any non-standard network attributes into TEAs
+          for(attr in TEAnetAttrs){
+            activate.network.attribute(base.net,attr,get.network.attribute(network.list[[i]],attr,unlist=FALSE),onset=onsets[i],terminus=termini[i])
+          }
         }
         
-        # TODO: copy any edge attributes into an edge TEA
       }
       
-      # TODO: copy any non-standard network attributes into network TEA
+      
       
     }
     ## check that net.obs.period has appropriate structure
@@ -483,25 +554,46 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
       
       # initialize
       #if (is.null(edge.spells)) activate.edges(base.net, onset=-Inf, terminus=Inf)
-      # assume edges in base.net to be active initially
-      if (!is.null(edge.toggles)) activate.edges(base.net, onset=-Inf, terminus=Inf)
       
-      for (i in seq_len(nrow(edge.data))) {
-        t <- edge.data[i,'tail'] 
-        h <- edge.data[i,'head']
-        e <- get.edgeIDs(base.net, t, h)
-        # add edge if not present in the base.net (as inactive?)
-        # TODO: problem here with directed vs undirected networks?
-        # TODO: how to handle multiplex/duplicate edge case?
-        if (length(e) == 0) {
-          add.edge(base.net, t, h)
-          e <- get.edgeIDs(base.net, t, h)
-          if (!is.null(edge.toggles)) deactivate.edges(base.net, e=e, onset=-Inf, terminus=Inf)
+      #if we are in the spells case, 
+      if (!is.null(edge.spells) ){
+        # if no edges exist yet we can avoid actually looping
+        if (network.edgecount(base.net) ==0){
+          dyads<-unique(edge.data[,3:4,drop=FALSE])
+          tails<-as.list(dyads[,1])
+          heads<-as.list(dyads[,2])
+          add.edges(base.net,tail=tails,head=heads)
+          eids<-sapply(seq_len(nrow(edge.data)),function(i){get.edgeIDs(base.net,v=edge.data[i,3],alter=edge.data[i,4])})
+          if (length(eids)>0){
+            activate.edges(base.net,onset=edge.data[,1],terminus=edge.data[,2],e=eids)
+          }
+        } else {  # there are pre-existing edges, so need to check for them while looping
+          for (i in seq_len(nrow(edge.data))){
+            eid<-get.edgeIDs(base.net,v=edge.data[i,3],alter=edge.data[i,4])
+            if (length(eid)==0){
+              # if the edge doesn't exist, add it and get the new edge id
+              add.edge(base.net,tail=edge.data[i,3],head=edge.data[i,4])
+              eid<-get.edgeIDs(base.net,v=edge.data[i,3],alter=edge.data[i,4])
+            } 
+            activate.edges(base.net,onset=edge.data[i,1],terminus=edge.data[i,2],e=eid)
+          }
         }
-        
-        if (!is.null(edge.spells)) {
-          activate.edges(base.net, e=e, onset=edge.data[i,'onset'], terminus=edge.data[i,'terminus'])
-        } else {
+  
+      } else {  # changes or toggles, so have to loop to avoid hurting our heads
+        # assume edges in base.net to be active initially
+        if (!is.null(edge.toggles)) activate.edges(base.net, onset=-Inf, terminus=Inf)
+        for (i in seq_len(nrow(edge.data))) {
+          t <- edge.data[i,2] 
+          h <- edge.data[i,3]
+          e <- get.edgeIDs(base.net, t, h)
+          # add edge if not present in the base.net (as inactive?)
+          # TODO: problem here with directed vs undirected networks?
+          # TODO: how to handle multiplex/duplicate edge case?
+          if (length(e) == 0) {
+            add.edge(base.net, t, h)
+            e <- get.edgeIDs(base.net, t, h)
+            if (!is.null(edge.toggles)) deactivate.edges(base.net, e=e, onset=-Inf, terminus=Inf)
+          }
           at <- edge.data[i,'time']
           change.activate <- (if (!is.null(edge.toggles)) !is.active(base.net, at=at, e=e) else edge.data[i,'direction']==1) 
           if (change.activate) {
@@ -509,8 +601,9 @@ networkDynamic <- function(base.net=NULL,edge.toggles=NULL,vertex.toggles=NULL,
           } else {
             deactivate.edges(base.net, e=e, onset=at, terminus=Inf)
           }
+          
         }
-      }
+      } # end of non-spell edge creation
     } # end edge data
     
   } # end non-network.list part
@@ -787,7 +880,6 @@ is.networkDynamic <- function(x){
 }
 
 as.networkDynamic <- function(object,...){
-  .Deprecated("networkDynamic()",old="as.networkDynamic.network",msg="networkDynamic converter and data merge functions have been refactored to avoid `as.networkDynamic.*' methods and S3 calls.  See networkDynamic function instead. The function as.networkDynamic.network will only set a networkDynamic class on its argument.")
   UseMethod("as.networkDynamic")
 }
 
