@@ -1,3 +1,12 @@
+#  Part of the statnet package, http://statnetproject.org
+#
+#  This software is distributed under the GPL-3 license.  It is free,
+#  open source, and has the attribution requirements (GPL Section 7) in
+#    http://statnetproject.org/attribution
+#
+#  Copyright 2013 the statnet development team
+######################################################################
+
 ########################################################################
 # This file contains the testing suite for the "query" methods, i.e.:
 #     is.active
@@ -307,6 +316,13 @@ if(any(g.results)){
 
 expect_equal(is.active(network.initialize(0),at=1),logical(0))
 
+# check for warning when eid specified for deleted edge
+test<-network.initialize(3)
+add.edges(test,1:3,c(2,3,1))
+delete.edges(test,eid=2)
+expect_warning(is.active(test,e=1:3,onset=-Inf,terminus=Inf),'correspond to deleted edges')
+expect_equal(suppressWarnings(is.active(test,e=1:3,onset=-Inf,terminus=Inf)),c(TRUE,TRUE))
+
 cat("ok\n")
 
 
